package org.javarosa.core.model.instance;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.javarosa.test.utils.ResourcePathHelper.r;
import static org.junit.Assert.assertEquals;

import java.io.FileInputStream;
import java.io.IOException;
import org.apache.commons.io.input.BOMInputStream;
import org.junit.Before;
import org.junit.Test;

public class CsvExternalInstanceTest {
    private TreeElement commaSeparated;
    private TreeElement semiColonSeparated;

    @Before
    public void setUp() throws IOException {
        commaSeparated = CsvExternalInstance.parse("id", r("external-secondary-comma-complex.csv").toString());
        semiColonSeparated = CsvExternalInstance.parse("id", r("external-secondary-semicolon-complex.csv").toString());
    }

    @Test
    public void heading_has_no_extra_quotes() {
        assertEquals("label", commaSeparated.getChildAt(0).getChildAt(0).getName());
        assertEquals("label", semiColonSeparated.getChildAt(0).getChildAt(0).getName());
    }

    @Test
    public void value_has_no_extra_quotes() {
        assertEquals("A", commaSeparated.getChildAt(0).getChildAt(0).getValue().getValue());
        assertEquals("A", semiColonSeparated.getChildAt(0).getChildAt(0).getValue().getValue());
    }

    @Test
    public void quoted_string_with_comma() {
        assertEquals("121 Main St, NE", commaSeparated.getChildAt(6).getChildAt(0).getValue().getValue());
        assertEquals("121 Main St, NE", semiColonSeparated.getChildAt(6).getChildAt(0).getValue().getValue());
    }

    @Test
    public void quoted_string_with_semicolon() {
        assertEquals("text; more text", commaSeparated.getChildAt(7).getChildAt(0).getValue().getValue());
        assertEquals("text; more text", semiColonSeparated.getChildAt(7).getChildAt(0).getValue().getValue());
    }

    @Test
    public void missing_fields_replaced_with_spaces() {
        for (int fieldIndex = 1; fieldIndex < 2; fieldIndex++) {
            assertEquals("", commaSeparated.getChildAt(5).getChildAt(fieldIndex).getValue().getValue());
            assertEquals("", semiColonSeparated.getChildAt(5).getChildAt(fieldIndex).getValue().getValue());
        }
    }

    @Test
    public void ignores_utf8_bom() throws IOException {
        BOMInputStream bomIs = new BOMInputStream(new FileInputStream(r("external-secondary-csv-bom.csv").toFile()));
        assertThat(bomIs.hasBOM(), is(true));

        TreeElement bomCsv = CsvExternalInstance.parse("id", r("external-secondary-csv-bom.csv").toString());
        assertThat(bomCsv.getChildAt(0).getChildAt(0).getName(), is("name"));
    }

    @Test
    public void parses_utf8_characters() throws IOException {
        TreeElement bomCsv = CsvExternalInstance.parse("id", r("external-secondary-csv-bom.csv").toString());
        assertThat(bomCsv.getChildAt(0).getChild("elevation", 0).getValue().getValue(), is("testé"));
    }
}
